import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/beans/video_tik_entry.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikPlayer {
  final VideoTikEntry tikEntry;

  final FijkPlayer player;

  final VideoPlayProgressModel progressModel;

  TikPlayer({required this.tikEntry, required this.player, required this.progressModel});
}

class VideoTikListModel extends CusChangeNotifier {
  PageController pageController = PageController();

  //当前索引
  ValueNotifier<int> index = ValueNotifier<int>(0);

  //播放列表
  List<TikPlayer> items = [];

  //是否前端显示页面
  bool isFrontPage = false;

  //获取播放器
  FijkPlayer? player(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.player;
  }

  //获取数据项
  VideoTikEntry? entry(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.tikEntry;
  }

  //播放进度
  VideoPlayProgressModel? progress(int position) {
    if (items.isEmptyList) {
      return null;
    }
    var item = items[position];
    return item.progressModel;
  }

  //获取当前播放器
  FijkPlayer? get currentPlayer => player(index.value);

  //播放视频数
  int get count => items.length;

  //视频翻页
  void videoTurn() {
    var p = pageController.page ?? 0;
    if (p % 1 == 0) {
      int target = p ~/ 1;
      if (index.value == target) {
        return;
      }
      var oldIndex = index.value;
      var newIndex = target;
      TikUtils.instance.recoveryPause(player(oldIndex));
      TikUtils.instance.start(player(newIndex));
      index.value = target;
    }
  }

  //添加视频
  appendVideos(List<VideoTikEntry> list) {
    for (var entry in list) {
      FijkPlayer player = FijkPlayer()
        ..setDataSource(
          entry.videoUrl,
          showCover: true,
          autoPlay: entry.autoPlay,
        );
      player.addListener(() => playerListener(list.indexOf(entry), player, entry));
      VideoPlayProgressModel progressModel = VideoPlayProgressModel();
      player.onCurrentPosUpdate.listen((event) => playProgressListen(player.value.duration, event, progressModel));
      var tikPlayer = TikPlayer(tikEntry: entry, player: player, progressModel: progressModel);
      items.add(tikPlayer);
    }
  }

  void playProgressListen(Duration total, Duration duration, VideoPlayProgressModel progressModel) {
    var currentTime = duration.inMilliseconds.toDouble();
    var totalTime = total.inMilliseconds.toDouble();
    var progress = currentTime / totalTime;
    progressModel.notifyProgress(progress);
    progressModel.notifierLazy();
  }

  void playerListener(int pos, FijkPlayer player, VideoTikEntry entry) {
    if (isFrontPage) {
      if (index.value == pos) {
        if (player.state == FijkState.completed) {
          player.start();
        }
      } else {
        TikUtils.instance.pause(player);
      }
    } else {
      TikUtils.instance.pause(player);
    }
  }

  //根据页面切换改变当前视频播放状态
  void changeCurrentPlayState() {
    if (isFrontPage) {
      TikUtils.instance.start(currentPlayer);
    } else {
      TikUtils.instance.pause(currentPlayer);
    }
  }

  //回收视频资源
  void release() {
    for (var item in items) {
      item.player.release();
    }
  }
}
