import 'dart:math';

import 'package:flutter/material.dart';

class FavoriteAnimationIcon extends StatefulWidget {
  /// 爱心偏移量
  final Offset position;

  /// 爱心大小
  final double size;

  /// 一次点赞完成回调
  final Function()? favoriteComplete;

  const FavoriteAnimationIcon({Key? key, required this.position, required this.size, this.favoriteComplete})
      : super(key: key);

  @override
  _FavoriteAnimationIconState createState() => _FavoriteAnimationIconState();
}

class _FavoriteAnimationIconState extends State<FavoriteAnimationIcon> with TickerProviderStateMixin {
  AnimationController? animationController;
  double appearDuration = 0.1;
  double dismissDuration = 0.8;

  @override
  void initState() {
    super.initState();
    animationController = AnimationController(
      lowerBound: 0,
      upperBound: 1,
      duration: const Duration(milliseconds: 500),
      vsync: this,
    );
    animationController?.addListener(() {
      setState(() {});
    });
    startAnimation();
  }

  startAnimation() async {
    await animationController?.forward();
    widget.favoriteComplete?.call();
  }

  double rotate = pi / 10.0 * (2 * Random().nextDouble() - 1);

  double get value => animationController?.value ?? 0;

  double get opa {
    if (value < appearDuration) {
      return 0.99 / appearDuration * value;
    }
    if (value < dismissDuration) {
      return 0.99;
    }
    var res = 0.99 - (value - dismissDuration) / (1 - dismissDuration);
    return res < 0 ? 0 : res;
  }

  double get scale {
    if (value < appearDuration) {
      return 1 + appearDuration - value;
    }
    if (value < dismissDuration) {
      return 1;
    }
    return (value - dismissDuration) / (1 - dismissDuration) + 1;
  }

  @override
  Widget build(BuildContext context) {
    return Positioned(
      child: Transform.rotate(
        angle: rotate,
        child: Opacity(
          opacity: opa,
          child: Transform.scale(
            alignment: Alignment.bottomCenter,
            scale: scale,
            child: ShaderMask(
              child: Icon(
                Icons.favorite,
                size: widget.size,
                color: Colors.redAccent,
              ),
              blendMode: BlendMode.srcATop,
              shaderCallback: (Rect bounds) => RadialGradient(
                center: Alignment.topLeft.add(const Alignment(0.66, 0.66)),
                colors: const [
                  Color(0xffEF6F6F),
                  Color(0xffF03E3E),
                ],
              ).createShader(bounds),
            ),
          ),
        ),
      ),
      left: widget.position.dx - widget.size / 2,
      top: widget.position.dy - widget.size / 2,
    );
  }

  @override
  void dispose() {
    animationController?.dispose();
    super.dispose();
  }
}
