import 'dart:async';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/tiklist/favorite_animation_icon.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';
import 'package:percent_indicator/percent_indicator.dart';

/// 单击：暂停；双击：点赞，双击后紧接着一次单击也是点赞；
class TikVideoGesture extends StatefulWidget {
  /// 子组件
  final Widget child;

  /// 播放图标单击事件
  final Function(VideoPlayerIconModel model) playIconTap;

  /// 点赞回调
  final Function()? favoriteCall;

  /// 单击回调
  final Function()? singleTapCall;

  /// 构建完成回调
  final Function() buildCall;

  /// 播放进度
  final VideoPlayProgressModel progressModel;

  const TikVideoGesture(
      {Key? key,
      required this.child,
      required this.playIconTap,
      this.favoriteCall,
      this.singleTapCall,
      required this.buildCall,
      required this.progressModel})
      : super(key: key);

  @override
  _TikVideoGestureState createState() => _TikVideoGestureState();
}

class _TikVideoGestureState extends State<TikVideoGesture> {
  List<Offset> icons = [];
  bool canAddFavorite = false;
  bool justAddFavorite = false;
  GlobalKey key = GlobalKey();
  Timer? timer;
  PortraitModel portraitModel = PortraitModel();
  VideoPlayerIconModel playerIconModel = VideoPlayerIconModel();

  @override
  void initState() {
    super.initState();
    widget.buildCall.call();
    CloudEBus.instance.addEvent(this, action: "playIconSetting", method: onPlayIconSetting);
  }

  @override
  void dispose() {
    super.dispose();
    CloudEBus.instance.dispatch("playIconSetting");
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: Stack(
        children: <Widget>[
          widget.child,
          buildFavoriteAnimationView(),
          buildRightView(),
          Positioned(
            child: buildPlayProgress(),
            left: 20,
            bottom: 80,
            right: 20,
          ),
        ],
      ),
      onTapDown: (detail) {
        setState(() {
          if (canAddFavorite) {
            icons.add(detail.globalPosition);
            widget.favoriteCall?.call();
            justAddFavorite = true;
          } else {
            justAddFavorite = false;
          }
        });
      },
      onTapUp: (detail) {
        timer?.cancel();
        var delay = canAddFavorite ? 500 : 600;
        timer = Timer(Duration(milliseconds: delay), () {
          canAddFavorite = false;
          timer = null;
          if (!justAddFavorite) {
            widget.singleTapCall?.call();
          }
        });
        canAddFavorite = true;
      },
    );
  }

  Widget buildPlayProgress() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayProgressModel>(widget.progressModel, (context, model) {
      return LinearPercentIndicator(
        width: MediaUtils.instance.screenWidth - 40,
        lineHeight: 2,
        percent: model.currentProgress,
        backgroundColor: Colors.white70,
        progressColor: Colors.black54,
        padding: EdgeInsets.zero,
      );
    });
  }

  Widget buildRightView() {
    return Container(
      alignment: Alignment.centerRight,
      child: Padding(
        padding: const EdgeInsets.only(right: 20, bottom: 60),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            buildPortrait(),
          ],
        ),
      ),
    );
  }

  Widget buildPortrait() {
    return StateUtils.instance.subscribeNotifierWidget<PortraitModel>(portraitModel, (context, model) {
      return Container(
        width: 56,
        height: 56,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.white, width: 0.8),
          borderRadius: const BorderRadius.all(Radius.circular(28)),
          color: Colors.white,
        ),
        child: Container(
          width: 42,
          height: 42,
          alignment: Alignment.center,
          child: Image.asset(
            "assets/images/ey_head_icon.png",
            package: "eyflutter_tik_video",
            width: 46,
            height: 46,
          ),
        ),
      );
    });
  }

  Widget buildFavoriteAnimationView() {
    return Stack(
      children: icons
          .map<Widget>(
            (p) => FavoriteAnimationIcon(
              key: Key(p.toString()),
              position: p,
              favoriteComplete: () {
                icons.remove(p);
              },
              size: 50,
            ),
          )
          .toList(),
    );
  }

  Widget buildPlayWidget() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayerIconModel>(playerIconModel, (context, model) {
      return Visibility(
        child: Align(
          alignment: Alignment.center,
          child: GestureDetector(
            child: Image.asset(
              "assets/images/ey_playing_icon.png",
              package: "eyflutter_tik_video",
              width: 58,
              height: 58,
            ),
            onTap: () => widget.playIconTap.call(model),
          ),
        ),
        visible: model.isDisplayPlayIcon,
      );
    });
  }

  void onPlayIconSetting(bool isHidePlayIcon) {
    if (isHidePlayIcon) {
      playerIconModel.isDisplayPlayIcon = false;
    } else {
      playerIconModel.isDisplayPlayIcon = true;
    }
    playerIconModel.notifierLazy();
  }
}
