import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/tiklist/tik_video_gesture.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikVideoView extends StatelessWidget {
  /// 播放组件
  final FijkPlayer player;

  /// 点赞回调
  final Function()? favoriteCall;

  /// 单击回调
  final Function()? singleTapCall;

  /// 构建完成回调
  final Function() buildCall;

  /// 播放进度
  final VideoPlayProgressModel progressModel;

  const TikVideoView(
      {Key? key,
      required this.player,
      this.favoriteCall,
      this.singleTapCall,
      required this.buildCall,
      required this.progressModel})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: <Widget>[
        TikVideoGesture(
          favoriteCall: favoriteCall,
          singleTapCall: singleTapCall,
          playIconTap: playIconTap,
          buildCall: buildCall,
          progressModel: progressModel,
          child: Stack(
            children: <Widget>[
              Container(
                height: double.infinity,
                width: double.infinity,
                alignment: Alignment.center,
                color: Colors.black,
                child: AspectRatio(
                  aspectRatio: 9 / 16.0,
                  child: buildVideo(),
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }

  Widget buildVideo() {
    return Center(
      child: FijkView(
        fit: FijkFit.fitHeight,
        player: player,
        color: Colors.black,
        panelBuilder: (_, __, ___, ____, _____) => Container(),
      ),
    );
  }

  void playIconTap(VideoPlayerIconModel model) {
    if (player.state == FijkState.started) {
      player.pause();
      model.isDisplayPlayIcon = true;
    } else {
      if (player.state == FijkState.prepared ||
          player.state == FijkState.paused ||
          player.state == FijkState.completed) {
        player.start();
        model.isDisplayPlayIcon = false;
      } else {
        model.isDisplayPlayIcon = true;
      }
    }
    model.notifierLazy();
  }
}
