import 'package:eyflutter_tik_video/models/video_tik_list_model.dart';
import 'package:eyflutter_tik_video/tiklist/tik_video_view.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class VideoTikListView extends StatefulWidget {
  /// 构建回调
  final Function(VideoTikListModel model) buildCall;

  const VideoTikListView({Key? key, required this.buildCall}) : super(key: key);

  @override
  _VideoTikListViewState createState() => _VideoTikListViewState();
}

class _VideoTikListViewState extends LifecycleState<VideoTikListView> {
  VideoTikListModel listModel = VideoTikListModel();

  @override
  String get viewName => "videoTikList";

  @override
  void initState() {
    super.initState();
    widget.buildCall.call(listModel);
    listModel.pageController.addListener(listModel.videoTurn);
  }

  @override
  void onPause(String routeName, Map<String, dynamic> arguments) {
    listModel.isFrontPage = false;
    TikUtils.instance.pause(listModel.currentPlayer);
  }

  @override
  void onResume(String routeName, Map<String, dynamic> arguments) {
    listModel.isFrontPage = true;
    TikUtils.instance.start(listModel.currentPlayer);
  }

  @override
  void onDestroy(String routeName, Map<String, dynamic> arguments) {
    listModel.release();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return StateUtils.instance.subscribeNotifierWidget<VideoTikListModel>(listModel, (context, model) {
      return PageView.builder(
        controller: model.pageController,
        pageSnapping: true,
        physics: const ClampingScrollPhysics(),
        scrollDirection: Axis.vertical,
        itemCount: model.count,
        itemBuilder: (context, i) {
          var player = model.player(i);
          var entry = model.entry(i);
          var progress = model.progress(i);
          return TikVideoView(
            key: Key("${entry?.videoUrl ?? ""}$i"),
            singleTapCall: () {
              if (player.state == FijkState.started) {
                TikUtils.instance.pause(player);
              } else {
                TikUtils.instance.start(player);
              }
              setState(() {});
            },
            favoriteCall: () {},
            player: player!,
            buildCall: () {},
            progressModel: progress!,
          );
        },
      );
    });
  }
}
